/*
 *  Simple test program -- demonstrate bug in IMFS not allowing
 *  readdir() to operate properly on root of mounted filesystem.
 */

#include <bsp.h>
#include <stdio.h>
#include <stdlib.h>

#include <dirent.h>
#include <assert.h>
#include <imfs.h>

#if defined(__rtems__)
#define d_type d_reclen
#endif

void printdir( const char *name )
{
  DIR *directory;
  struct dirent *d;
  int rc;

  printf("Listing of directory %s\n", name );
  directory = opendir(name);
  assert( directory );

  printf( "    %-20s %8s %8s %8s %4s\n",
     "     name", "inode", " offset", "reclen", " type" );
  d = readdir(directory);
  while (d) {
    printf( "    %-20s %8d %8d %6d   0x%04x\n",
       d->d_name, (int)d->d_ino, (int)d->d_off, d->d_reclen, d->d_type );
    d = readdir(directory);
  }
  rc = closedir(directory);
  assert( !rc );
  printf("End of %s\n", name );
}             

rtems_task Init(
  rtems_task_argument ignored
)
{
  rtems_filesystem_mount_table_entry_t *entry;
  int rc;

  puts( "\n\n*** MOUNT DEMO ***" );

  puts("Mounting secondary IMFS (at /imfs)...");
  rc = mkdir("/imfs", S_IRWXU | S_IRWXG | S_IRWXO);
  assert( !rc  );
  rc = mkdir("/imfs/should_be_hidden", S_IRWXU | S_IRWXG | S_IRWXO);
  assert( !rc  );

  rc = mount(&entry, &IMFS_ops, RTEMS_FILESYSTEM_READ_ONLY, NULL, "/imfs");
  assert( !rc  );

  rc = mkdir("/imfs/testdir", S_IRWXU | S_IRWXG | S_IRWXO);
  assert( !rc  );

  rc = mkdir("/imfs/testdir/testsubdir", S_IRWXU | S_IRWXG | S_IRWXO);
  assert( !rc  );

  printdir( "/imfs" );
  printdir( "/imfs/" );
  printdir( "/imfs/." );
  printdir( "/imfs/testdir" );
  printdir( "/imfs/testdir/.." );

  printf( "*** END OF MOUNT DEMO ***\n" );
  exit( 0 );
}

/* configuration information */

#define CONFIGURE_TEST_NEEDS_CONSOLE_DRIVER
#define CONFIGURE_RTEMS_INIT_TASKS_TABLE
#define CONFIGURE_USE_IMFS_AS_BASE_FILESYSTEM
#define CONFIGURE_LIBIO_MAXIMUM_FILE_DESCRIPTORS 10
#define CONFIGURE_MAXIMUM_TASKS 1

#define CONFIGURE_INIT

#include <confdefs.h>

/* end of file */
